from ..base import _call

'''
All api requests under the tv seasons tab in https://developers.themoviedb.org/3/tv-seasons
'''


def details(tv_id, season_number, **kwargs):
    '''
    Get the TV season details by id.
    Supports append_to_response. https://developers.themoviedb.org/3/getting-started/append-to-response

    required: tv_id, season_number
    optional: language, append_to_response
    '''

    return _call('GET', f'https://api.themoviedb.org/3/tv/{tv_id}/season/{season_number}', params=kwargs)


def changes(season_id, **kwargs):
    '''
    Get the changes for a TV season. By default only the last 24 hours are returned.

    You can query up to 14 days in a single query by using the start_date and end_date query parameters.

    required: season_id
    optional: page, start_date, end_date
    '''

    return _call('GET', f'https://api.themoviedb.org/3/tv/season/{season_id}/changes', params=kwargs)


def account_states(tv_id, season_number, **kwargs):
    '''
    Returns all of the user ratings for the season's episodes.

    required: tv_id, season_number, session_id or guest_session_id
    optional: language
    '''

    return _call('GET', f'https://api.themoviedb.org/3/tv/{tv_id}/season/{season_number}/account_states', params=kwargs)


def credits(tv_id, season_number, **kwargs):
    '''
    Get the credits (cast and crew) that have been added to a TV show.

    required: tv_id, season_number
    optional: language
    '''

    return _call('GET', f'https://api.themoviedb.org/3/tv/{tv_id}/season/{season_number}/credits', params=kwargs)


def external_ids(tv_id, season_number, **kwargs):
    '''
    Get the external ids for a TV season. We currently support the following external sources.

    Media Databases
        TVDB ID
        Freebase MID*
        Freebase ID*
        TVRage ID*

    *Defunct or no longer available as a service.

    required: tv_id, season_number
    optional: language
    '''

    return _call('GET', f'https://api.themoviedb.org/3/tv/{tv_id}/season/{season_number}/external_ids', params=kwargs)


def images(tv_id, season_number, **kwargs):
    '''
    Get the images that belong to a TV season.
    Querying images with a language parameter will filter the results.
    If you want to include a fallback language (especially useful for backdrops) you can use the include_image_language parameter.
    This should be a comma seperated value like so: include_image_language=en,null.

    required: tv_id, season_number
    optional: language
    '''

    return _call('GET', f'https://api.themoviedb.org/3/tv/{tv_id}/season/{season_number}/images', params=kwargs)


def videos(tv_id, season_number, **kwargs):
    '''
    Get the videos that have been added to a TV season.

    required: tv_id, season_number
    optional: language
    '''

    return _call('GET', f'https://api.themoviedb.org/3/tv/{tv_id}/season/{season_number}/videos', params=kwargs)



from ..base import _call

'''
All api requests under the guest sessions tab in https://developers.themoviedb.org/3/guest-sessions
'''

def rated_movies(guest_session_id, **kwargs):
    '''
    Get the rated movies for a guest session.

    required: guest_session_id
    optional: language, sort_by
    '''

    return _call('GET', f'https://api.themoviedb.org/3/guest_session/{guest_session_id}/rated/movies', params=kwargs)

def rated_tv_shows(guest_session_id, **kwargs):
    '''
    Get the rated TV shows for a guest session.

    required: guest_session_id
    optional: language, sort_by
    '''

    return _call('GET', f'https://api.themoviedb.org/3/guest_session/{guest_session_id}/rated/tv', params=kwargs)

def rated_tv_episodes(guest_session_id, **kwargs):
    '''
    Get the rated TV episodes for a guest session.

    required: guest_session_id
    optional: language, sort_by
    '''

    return _call('GET', f'https://api.themoviedb.org/3/guest_session/{guest_session_id}/rated/tv/episodes', params=kwargs)



from ..base import _call

'''
All api requests under the authentication tab in https://developers.themoviedb.org/3/authentication

https://developers.themoviedb.org/3/authentication/how-do-i-generate-a-session-id
'''

def create_guest_session():
    '''
    This method will let you create a new guest session.
    Guest sessions are a type of session that will let a user rate movies and TV shows but not require them to have a TMDb user account.
    More information about user authentication can be found https://developers.themoviedb.org/3/authentication/how-do-i-generate-a-session-id.

    Please note, you should only generate a single guest session per user (or device) as you will be able to attach the ratings to a TMDb user account in the future.
    There is also IP limits in place so you should always make sure it's the end user doing the guest session actions.

    If a guest session is not used for the first time within 24 hours, it will be automatically deleted.

    required:
    optional:
    '''

    return _call('GET', f'https://api.themoviedb.org/3/authentication/guest_session/new')

def create_request_token():
    '''
    Create a temporary request token that can be used to validate a TMDb user login. More details about how this works can be found https://developers.themoviedb.org/3/authentication/how-do-i-generate-a-session-id.

    required:
    optional:
    '''

    return _call('GET', f'https://api.themoviedb.org/3/authentication/token/new')

def create_request_token(request_token):
    '''
    You can use this method to create a fully valid session ID once a user has validated the request token.
    More information about how this works can be found https://developers.themoviedb.org/3/authentication/how-do-i-generate-a-session-id.

    required: request_token
    optional:
    '''

    payload = {
        'request_token': request_token
    }

    return _call('POST', f'https://api.themoviedb.org/3/authentication/session/new', payload=payload)

def create_request_token(username, password, request_token):
    '''
    This method allows an application to validate a request token by entering a username and password.

    Not all applications have access to a web view so this can be used as a substitute.

    Please note, the preferred method of validating a request token is to have a user authenticate the request via the TMDb website.
    You can read about that method https://developers.themoviedb.org/3/authentication/how-do-i-generate-a-session-id.

    If you decide to use this method please use HTTPS.

    required: username, password, request_token
    optional:
    '''

    payload = {
        'username': username,
        'password': password,
        'request_token': request_token
    }

    return _call('POST', f'https://api.themoviedb.org/3/authentication/token/validate_with_login', payload=payload)

def create_request_token(access_token):
    '''
    Use this method to create a v3 session ID if you already have a valid v4 access token.
    The v4 token needs to be authenticated by the user.
    Your standard "read token" will not validate to create a session ID.

    required: access_token
    optional:
    '''

    payload = {
        'access_token': access_token
    }

    return _call('POST', f'https://api.themoviedb.org/3/authentication/session/convert/4', payload=payload)

def create_request_token(session_id):
    '''
    If you would like to delete (or "logout") from a session, call this method with a valid session ID.

    required: session_id
    optional:
    '''

    payload = {
        'session_id': session_id
    }

    return _call('DELETE', f'https://api.themoviedb.org/3/authentication/session', payload=payload)



from ..base import _call

'''
All api requests under the reviews tab in https://developers.themoviedb.org/3/reviews
'''

def details(review_id):
    '''

    required: review_id
    optional:
    '''

    return _call('GET', f'https://api.themoviedb.org/3/review/{review_id}')



from ..base import _call

'''
All api requests under the list tab in https://developers.themoviedb.org/4/list
'''

def list(list_id, access_token, **kwargs):
    '''
    Get the details of a list.

    required: list_id
    optional: page, language, sort_by
    '''

    return _call('GET', f'https://api.themoviedb.org/4/list/{list_id}', params=kwargs, bearer=access_token)

def create_list(access_token, payload):
    '''
    This method will create a new list.
    You will need to have valid user access token in order to create a new list.

    required: payload 
    optional: 
    '''

    # payload = {
    #     "name": name,
    #     "iso_639_1": iso,
    #     'description': description,
    #     'public': public
    #     }

    return _call('POST', f'https://api.themoviedb.org/4/list', bearer=access_token, payload=payload)

def update_list(access_token, list_id, payload):
    '''
    This method will let you update the details of a list.
    You must be the owner of the list and therefore have a valid user access token in order to edit it.

    required: list_id, payload
    optional: 
    '''
    # payload = {
    #     'name': name,
    #     'description': description,
    #     'language': language
    #     }

    return _call('PUT', f'https://api.themoviedb.org/4/list/{list_id}', bearer=access_token, payload=payload)

def clear_list(access_token, list_id):
    '''
    This method lets you clear all of the items from a list in a single request.
    This action cannot be reversed so use it with caution.
    You must be the owner of the list and therefore have a valid user access token in order to clear a list.

    required: list_id
    optional: 
    '''

    return _call('GET', f'https://api.themoviedb.org/4/list/{list_id}/clear', bearer=access_token)

def delete_list(access_token, list_id):
    '''
    This method will delete a list by id. This action is not reversible so take care when issuing it.
    You must be the owner of the list and therefore have a valid user access token in order to delete it.

    required: list_id, session_id, media_id
    optional: 
    '''

    return _call('DELETE', f'https://api.themoviedb.org/4/list/{list_id}', bearer=access_token)

def add_items(access_token, list_id, payload):
    '''
    This method will let you add items to a list. We support essentially an unlimited number of items to be posted at a time.
    Both movie and TV series are support.

    The results of this query will return a results array.
    Each result includes a success field. If a result is false this will usually indicate that the item already exists on the list.
    It may also indicate that the item could not be found.

    You must be the owner of the list and therefore have a valid user access token in order to add items to a list.

    required: list_id, payload
    optional:
    '''

    # payload = {
    #     example payload
    # }

    return _call('POST', f'https://api.themoviedb.org/4/list/{list_id}/items', bearer=access_token, payload=payload)

def update_items(access_token, list_id, payload):
    '''
    This method will let you update an individual item on a list. Currently, only adding a comment is suported.

    You must be the owner of the list and therefore have a valid user access token in order to edit items.

    required: list_id, payload
    optional: 
    '''

    return _call('PUT', f'https://api.themoviedb.org/4/list/{list_id}/items', bearer=access_token, payload=payload)

def remove_items(access_token, list_id, payload):
    '''
    This method will let you remove items from a list. You can remove multiple items at a time.

    You must be the owner of the list and therefore have a valid user access token in order to delete items from it.

    required: list_id, payload
    optional: 
    '''

    return _call('DELETE', f'https://api.themoviedb.org/4/list/{list_id}/items', bearer=access_token, payload=payload)

def check_item_status(access_token, list_id, **kwargs):
    '''
    This method lets you quickly check if the item is already added to the list.

    You must be the owner of the list and therefore have a valid user access token in order to check an item status.

    required: list_id, media_id, media_type
    optional:
    '''

    return _call('DELETE', f'https://api.themoviedb.org/4/list/{list_id}/item_status', params=kwargs, bearer=access_token)



from ..base import _call

'''
All api requests under the cerifications tab in https://developers.themoviedb.org/3/cerifications
'''

def movie():
    '''
    Get an up to date list of the officially supported movie certifications on TMDb.

    required: 
    optional:
    '''

    return _call('GET', f'https://api.themoviedb.org/3/certification/movie/list')

def tv():
    '''
    Get an up to date list of the officially supported TV show certifications on TMDb.

    required: 
    optional:
    '''

    return _call('GET', f'https://api.themoviedb.org/3/certification/tv/list')



from ..base import _call

'''
All api requests under the trending tab in https://developers.themoviedb.org/3/trending
'''

def trending(media_type, time_window):
    '''
    Get the daily or weekly trending items.
    The daily trending list tracks items over the period of a day while items have a 24 hour half life.
    The weekly list tracks items over a 7 day period, with a 7 day half life.

    Valid Media Types
    Media Type      Description
    all             Include all movies, TV shows and people in the results as a global trending list.
    movie           Show the trending movies in the results.
    tv              Show the trending TV shows in the results.
    person          Show the trending people in the results.

    Valid Time Windows
    Time Window     Description
    day             View the trending list for the day.
    week            View the trending list for the week.

    required:
    optional:
    '''

    return _call('GET', f'https://api.themoviedb.org/3/trending/{media_type}/{time_window}')



from ..base import _call

'''
All api requests under the changes tab in https://developers.themoviedb.org/3/changes
'''

def movie(**kwargs):
    '''
    Get a list of all of the movie ids that have been changed in the past 24 hours.
    You can query it for up to 14 days worth of changed IDs at a time with the start_date and end_date query parameters.
    100 items are returned per page.

    required: 
    optional: page, start_date, end_date
    '''

    return _call('GET', f'https://api.themoviedb.org/3/movie/changes', params=kwargs)

def tv(**kwargs):
    '''
    Get a list of all of the TV show ids that have been changed in the past 24 hours.
    You can query it for up to 14 days worth of changed IDs at a time with the start_date and end_date query parameters.
    100 items are returned per page.

    required: 
    optional: page, start_date, end_date
    '''

    return _call('GET', f'https://api.themoviedb.org/3/tv/changes', params=kwargs)

def person(**kwargs):
    '''
    Get a list of all of the person ids that have been changed in the past 24 hours.
    You can query it for up to 14 days worth of changed IDs at a time with the start_date and end_date query parameters.
    100 items are returned per page.

    required: 
    optional: page, start_date, end_date
    '''

    return _call('GET', f'https://api.themoviedb.org/3/person/changes', params=kwargs)